import os
import math

from lib.config import *

from lxml import etree


def check_stl(filename, module):

    assert filename[-1] == 'b'

    scad = "../scad/tmp.scad"

    with open(scad, "w") as fd:
        fd.write("include <model.scad>;\n")
        fd.write(module + "();")

    tmp_stl = "../stl/tmp.stl"

    os.system(" ".join(['openscad', '-o', tmp_stl, scad]))

    os.system("ivcon {} {}".format(tmp_stl, filename))

    os.remove(tmp_stl)
    os.remove(scad)


def generate_link(robot, name, scad_module):
    link = etree.SubElement(robot, 'link', name=name)

    visual = etree.SubElement(link, 'visual')

    geometry = etree.SubElement(visual, 'geometry')

    if scad_module is not None:
        stl_filename = 'stl/' + name + ".stlb"

        if not os.path.exists("../" + stl_filename):
            check_stl("../" + stl_filename, scad_module)

        etree.SubElement(
            geometry, 'mesh', filename="package://schpin_tote/" + stl_filename)

        material = etree.SubElement(
            visual, 'material', name=name + "_material")

        etree.SubElement(
            material, 'color', rgba="{} {} {} 1".format(*get_color(name)))

    return link


def get_color(name):
    m = hashlib.md5()
    m.update(name.encode('utf8'))
    h = m.digest()

    return [x / 256 for x in h[:3]]


def generate_urdf_joint(robot,
                        name,
                        parent,
                        child,
                        pos,
                        rot,
                        axis,
                        static=False):
    joint = etree.SubElement(robot, 'joint', name=name, type='revolute')

    etree.SubElement(joint, 'parent', link=parent.name)
    etree.SubElement(joint, 'child', link=child.name)

    if not static:
        etree.SubElement(
            joint,
            'limit',
            effort="1000",
            velocity="50",
            lower="-2",
            upper="2")

    etree.SubElement(
        joint,
        'origin',
        xyz="{} {} {}".format(* [x for x in pos]),
        rpy="{} {} {}".format(* [math.radians(x) for x in rot]))

    if axis is not None:
        etree.SubElement(joint, 'axis', xyz="{} {} {}".format(*axis))


" return number of replaced items, unreplaced items, new value "
